import sequelizeConnection from '../config'
import {DataTypes, Model, Optional} from "sequelize";

interface UserAttributes {
    id: number;
    name: string;
    isBootsWart: boolean;
    isCoordinator: boolean;
    email: string;
    notifications: boolean
}
export interface UserInput extends Optional<UserAttributes, 'id'> {}
export interface UserOutput extends Required<UserAttributes> {}
class User extends Model<UserAttributes> implements UserAttributes {
    public id!: number;
    public name!: string;
    public isBootsWart: boolean;
    public isCoordinator: boolean;
    public email: string;
    public notifications: boolean;

    public readonly createdAt!: Date;
    public readonly updatedAt!: Date;
}
User.init(
    {
        id: {
            type: DataTypes.INTEGER,
            autoIncrement: true,
            primaryKey: true,
        },
        name: {
            type: new DataTypes.STRING,
            allowNull: false,
        },
        isBootsWart: {
            type: DataTypes.BOOLEAN,
            allowNull: false,
        },
        isCoordinator: {
            type: DataTypes.BOOLEAN,
            allowNull: false,
        },
        email: {
            type: new DataTypes.STRING,
            allowNull: false,
        },
        notifications: {
            type: DataTypes.BOOLEAN,
            allowNull: false,
        }
    },
    {
        tableName: "users",
        sequelize: sequelizeConnection,
    }
);

export const create = async (payload: User): Promise<UserOutput> => {
    return await User.create(payload)
}
export const update = async (id: number, payload: Partial<UserInput>): Promise<UserOutput> => {
    const user = await User.findByPk(id)
    if (!user) {
        throw new Error('not found')
    }
    return await (user as User).update(payload)
}
export const getById = async (id: number): Promise<UserInput> => {
    const user = await User.findByPk(id)
    if (!user) {
        throw new Error('not found')
    }
    return user
}
const deleteById = async (id: number): Promise<boolean> => {
    const deletedUserCount = await User.destroy({
        where: {id}
    })
    return !!deletedUserCount
}
export const getAll = async (): Promise<UserOutput[]> => {
    return User.findAll()
}
export const UserService = {
    create, update, getById, deleteById, getAll
}

export default User
